#!/usr/bin/ruby

@base_dir = Dir::getwd + "/" + File.dirname($0)

@module_name = "redundancy_solution_module"

@tarballs = [
  "PA10.rtc",
  "arm_moving_module",
  "direction_scenario",
  "redundancy_elbow_angle",
  "redundancy_maximize_manipulability",
  "redundancy_minimize_norm"
]

@other_files = [
  "README.txt", 
  "Makefile", 
  "build.rb"
]


def exec(cmd)
  puts "<execute> #{cmd}"
  system(cmd)
end

def exec_async(cmd)
  puts "<execute async> #{cmd}"
  system(cmd + " &")
end

def extract
  @tarballs.each {|f|
    Dir::chdir(@base_dir)
    exec("tar xvfz #{f}-*tar.gz")
  }
end

def _recursive_exec(cmd, d)
  Dir::chdir(d)

  Dir.glob("*") {|f|
    _recursive_exec(cmd, f) if File::ftype(f) == "directory"
  }

  if File.exist? "Makefile"
    exec(cmd)
  end

  Dir::chdir("..")
end

def make
  @tarballs.each {|d|
    Dir::chdir(@base_dir)
    _recursive_exec("make", d)
  }
end

def clean
  @tarballs.each {|d|
    Dir::chdir(@base_dir)
    _recursive_exec("make clean", d)
  }
end

def tarball
  clean

  datestr = Time.now.strftime("%Y%m%d")

  @tarballs.each {|f|
    Dir::chdir(@base_dir)
    Dir::chdir(f)
    if File.exist? "Makefile"
      system("make clean")
    end

    Dir::chdir(@base_dir)

    cmd = "tar cvfz #{f}-#{datestr}.tar.gz --exclude=.svn #{f}"
    puts cmd
    system(cmd)

  }

  Dir::chdir(@base_dir)
  tmpdir = "/tmp/#{@module_name}-#{datestr}"
  exec "mkdir -p #{tmpdir}"
  exec "mv *.tar.gz #{tmpdir}"
  @other_files.each {|f|
    exec "cp #{f} #{tmpdir}"
  }

  Dir::chdir(tmpdir)
  Dir::chdir("..")
  exec "tar cvfz #{@module_name}-#{datestr}.tar.gz #{@module_name}-#{datestr}" 
  exec "mv #{@module_name}-#{datestr}.tar.gz ~"
  exec "rm -rf #{tmpdir}"
end

#
#
#
@activate_modules = [
  ["./arm_moving_module/", "ArmMovingModuleComp"],
  ["./direction_scenario/", "DirectionScenarioComp"],
]

def _setup_modules
  p @activate_modules
  @activate_modules.each {|a|
    Dir.chdir(@base_dir)
    Dir.chdir(a[0])
    exec_async("make run")
  }
  Dir.chdir(@base_dir)
end

def _connect_module(m)
  conn = [
    ["ArmMovingModule", "q_cur_out", m, "q_cur_in"],
    [m, "xyzrpy_out", "DirectionScenario", "xyzrpy_in"],
    ["DirectionScenario", "xyzrpy_out", m, "xyzrpy_in"],
    [m, "q_ref_out", "ArmMovingModule", "q_ref_in"]
  ]

  ns = "/127.0.0.1:2809/"

  conn.each {|a|
    cmd = "rtcon #{ns}#{a[0]}0.rtc:.#{a[1]} #{ns}#{a[2]}0.rtc:.#{a[3]}"
    exec(cmd)
  }
end

def _activate_module(rtc)
  cmd = "rtact /localhost:2809/" + rtc
  exec(cmd)
end

def setup_redundancy_minimize_norm
  rtc  = "RedundancyMinimizeNorm"
  path = "./redundancy_minimize_norm/"
  @activate_modules << [path, rtc + "Comp"]
  _setup_modules
  _connect_module(rtc)
  _activate_module(rtc + "0.rtc")
end

def setup_redundancy_elbow_angle
  rtc  = "RedundancyElbowAngle"
  path = "./redundancy_elbow_angle/"
  @activate_modules << [path, rtc + "Comp"]
  _setup_modules
  _connect_module(rtc)
  _activate_module(rtc + "0.rtc")
end

def setup_redundancy_maximize_manipulability
  rtc  = "RedundancyMaxmizeManipulaibility"
  path = "./redundancy_maximize_manipulability/"
  @activate_modules << [path, rtc + "Comp"]
  _setup_modules
  _connect_module(rtc)
  _activate_module(rtc + "0.rtc")
end

def shutdown
  @activate_modules << ["./redundancy_minimize_norm/","RedundancyMinimizeNormComp"]
  @activate_modules << ["./redundancy_maximize_manipulability/","RedundancyMaxmizeManipulaibilityComp"]
  @activate_modules << ["./redundancy_elbow_angle/","RedundancyElbowAngleComp"]

  @activate_modules.each{|a|
    cmd = "rtexit /localhost:2809/" + a[1].gsub(/Comp/,"") + "0.rtc";
    exec(cmd)
  }
end

def usage
  puts "usage : build.sh {extract|make|run|tarball|clean}"
  exit 0
end

if __FILE__ == $0
  usage if ARGV.size != 1

  puts "execute cmd=#{ARGV[0]}"
  puts "base_dir = #{@base_dir}"
  Dir::chdir(@base_dir)

  send(ARGV[0])
end
